Package: PKNCA
Type: Package
Title: Perform Pharmacokinetic Non-Compartmental Analysis
Version: 0.12.1
Authors@R: c(
    person("Bill", "Denney", email="wdenney@humanpredictions.com", role=c("aut", "cre"), comment=c(ORCID="0000-0002-5759-428X")),
    person("Clare", "Buckeridge", email="clare.buckeridge@pfizer.com", role="aut"),
    person("Gerardo Jose", "Rodriguez", email="gerardo.jrac@gmail.com", role="aut", comment=c(ORCID="0000-0003-1413-0060")),
    person("Sridhar", "Duvvuri", role="ctb"))
Imports: checkmate, dplyr (>= 0.5.0), digest, nlme, purrr, rlang,
        stats, tidyr, tibble, utils, lifecycle
Suggests: covr, cowplot, ggplot2, knitr, labeling, pander, pmxTools,
        rmarkdown, spelling, testthat (>= 3.0.0), units, withr
Description: Compute standard Non-Compartmental Analysis (NCA) parameters for
    typical pharmacokinetic analyses and summarize them.
License: AGPL-3
URL: https://humanpred.github.io/pknca/,
        https://github.com/humanpred/pknca,
        http://humanpred.github.io/pknca/
BugReports: https://github.com/humanpred/pknca/issues
NeedsCompilation: no
VignetteBuilder: knitr
RoxygenNote: 7.3.2
Config/testthat/edition: 3
Encoding: UTF-8
Language: en-US
Packaged: 2025-08-19 00:38:28 UTC; bill
Author: Bill Denney [aut, cre] (ORCID: <https://orcid.org/0000-0002-5759-428X>),
  Clare Buckeridge [aut],
  Gerardo Jose Rodriguez [aut] (ORCID:
    <https://orcid.org/0000-0003-1413-0060>),
  Sridhar Duvvuri [ctb]
Maintainer: Bill Denney <wdenney@humanpredictions.com>
Repository: CRAN
Date/Publication: 2025-08-19 04:40:02 UTC
Built: R 4.6.0; ; 2026-04-26 16:06:45 UTC; unix
