# Generated by roxygen2: do not edit by hand

S3method(PKNCAconc,data.frame)
S3method(PKNCAconc,default)
S3method(PKNCAconc,tbl_df)
S3method(PKNCAdata,PKNCAconc)
S3method(PKNCAdata,PKNCAdose)
S3method(PKNCAdata,default)
S3method(PKNCAdose,data.frame)
S3method(PKNCAdose,default)
S3method(PKNCAdose,tbl_df)
S3method(as.data.frame,PKNCAconc)
S3method(as.data.frame,PKNCAdose)
S3method(as.data.frame,PKNCAresults)
S3method(as_PKNCAconc,PKNCAconc)
S3method(as_PKNCAconc,PKNCAdata)
S3method(as_PKNCAconc,PKNCAresults)
S3method(as_PKNCAdata,PKNCAdata)
S3method(as_PKNCAdata,PKNCAresults)
S3method(as_PKNCAdose,PKNCAdata)
S3method(as_PKNCAdose,PKNCAdose)
S3method(as_PKNCAdose,PKNCAresults)
S3method(as_PKNCAresults,PKNCAdata)
S3method(as_PKNCAresults,PKNCAresults)
S3method(dplyr::group_vars,PKNCAconc)
S3method(dplyr::group_vars,PKNCAdata)
S3method(dplyr::group_vars,PKNCAdose)
S3method(dplyr::group_vars,PKNCAresults)
S3method(exclude,default)
S3method(filter,PKNCAconc)
S3method(filter,PKNCAdose)
S3method(filter,PKNCAresults)
S3method(formula,PKNCAconc)
S3method(formula,PKNCAdose)
S3method(full_join,PKNCAconc)
S3method(full_join,PKNCAdose)
S3method(full_join,PKNCAresults)
S3method(getDataName,PKNCAconc)
S3method(getDataName,PKNCAdose)
S3method(getDataName,PKNCAresults)
S3method(getDepVar,PKNCAconc)
S3method(getDepVar,PKNCAdose)
S3method(getGroups,PKNCAconc)
S3method(getGroups,PKNCAdata)
S3method(getGroups,PKNCAdose)
S3method(getGroups,PKNCAresults)
S3method(getIndepVar,PKNCAconc)
S3method(getIndepVar,PKNCAdose)
S3method(group_by,PKNCAconc)
S3method(group_by,PKNCAdose)
S3method(group_by,PKNCAresults)
S3method(inner_join,PKNCAconc)
S3method(inner_join,PKNCAdose)
S3method(inner_join,PKNCAresults)
S3method(is_sparse_pk,PKNCAconc)
S3method(is_sparse_pk,PKNCAdata)
S3method(is_sparse_pk,PKNCAresults)
S3method(left_join,PKNCAconc)
S3method(left_join,PKNCAdose)
S3method(left_join,PKNCAresults)
S3method(model.frame,PKNCAconc)
S3method(model.frame,PKNCAdose)
S3method(mutate,PKNCAconc)
S3method(mutate,PKNCAdose)
S3method(mutate,PKNCAresults)
S3method(print,PKNCAconc)
S3method(print,PKNCAdata)
S3method(print,PKNCAdose)
S3method(print,provenance)
S3method(print,summary_PKNCAresults)
S3method(right_join,PKNCAconc)
S3method(right_join,PKNCAdose)
S3method(right_join,PKNCAresults)
S3method(setDuration,PKNCAconc)
S3method(setDuration,PKNCAdose)
S3method(setRoute,PKNCAdose)
S3method(signifString,data.frame)
S3method(signifString,default)
S3method(summary,PKNCAconc)
S3method(summary,PKNCAdata)
S3method(summary,PKNCAdose)
S3method(summary,PKNCAresults)
S3method(superposition,PKNCAconc)
S3method(superposition,numeric)
S3method(time_calc,POSIXt)
S3method(time_calc,difftime)
S3method(time_calc,numeric)
S3method(ungroup,PKNCAconc)
S3method(ungroup,PKNCAdose)
S3method(ungroup,PKNCAresults)
export(PKNCA.choose.option)
export(PKNCA.options)
export(PKNCA.options.describe)
export(PKNCA.set.summary)
export(PKNCA_impute_method_start_cmin)
export(PKNCA_impute_method_start_conc0)
export(PKNCA_impute_method_start_predose)
export(PKNCAconc)
export(PKNCAdata)
export(PKNCAdose)
export(PKNCAresults)
export(add.interval.col)
export(addProvenance)
export(adj.r.squared)
export(as_PKNCAconc)
export(as_PKNCAdata)
export(as_PKNCAdose)
export(as_PKNCAresults)
export(as_sparse_pk)
export(assert_intervals)
export(business.cv)
export(business.geocv)
export(business.geomean)
export(business.max)
export(business.mean)
export(business.median)
export(business.min)
export(business.range)
export(business.sd)
export(check.conc.time)
export(check.conversion)
export(check.interval.specification)
export(checkProvenance)
export(choose.auc.intervals)
export(clean.conc.blq)
export(clean.conc.na)
export(cov_holder)
export(exclude)
export(exclude_nca_by_param)
export(exclude_nca_count_conc_measured)
export(exclude_nca_max.aucinf.pext)
export(exclude_nca_min.hl.adj.r.squared)
export(exclude_nca_min.hl.r.squared)
export(exclude_nca_span.ratio)
export(exclude_nca_tmax_0)
export(exclude_nca_tmax_early)
export(extrapolate.conc)
export(filter)
export(find.tau)
export(full_join)
export(geocv)
export(geomean)
export(geosd)
export(get.best.model)
export(get.interval.cols)
export(get.parameter.deps)
export(getDepVar)
export(getGroups)
export(getIndepVar)
export(get_halflife_points)
export(group_by)
export(inner_join)
export(interp.extrap.conc)
export(interp.extrap.conc.dose)
export(interpolate.conc)
export(is_sparse_pk)
export(left_join)
export(mutate)
export(pk.business)
export(pk.calc.ae)
export(pk.calc.auc)
export(pk.calc.auc.all)
export(pk.calc.auc.inf)
export(pk.calc.auc.inf.obs)
export(pk.calc.auc.inf.pred)
export(pk.calc.auc.last)
export(pk.calc.aucabove)
export(pk.calc.aucint)
export(pk.calc.aucint.all)
export(pk.calc.aucint.inf.obs)
export(pk.calc.aucint.inf.pred)
export(pk.calc.aucint.last)
export(pk.calc.auciv)
export(pk.calc.auciv_pbext)
export(pk.calc.aucpext)
export(pk.calc.aumc)
export(pk.calc.aumc.all)
export(pk.calc.aumc.inf)
export(pk.calc.aumc.inf.obs)
export(pk.calc.aumc.inf.pred)
export(pk.calc.aumc.last)
export(pk.calc.auxc)
export(pk.calc.c0)
export(pk.calc.cav)
export(pk.calc.ceoi)
export(pk.calc.cl)
export(pk.calc.clast.obs)
export(pk.calc.clr)
export(pk.calc.cmax)
export(pk.calc.cmin)
export(pk.calc.count_conc)
export(pk.calc.count_conc_measured)
export(pk.calc.cstart)
export(pk.calc.ctrough)
export(pk.calc.deg.fluc)
export(pk.calc.dn)
export(pk.calc.f)
export(pk.calc.fe)
export(pk.calc.half.life)
export(pk.calc.kel)
export(pk.calc.mrt)
export(pk.calc.mrt.iv)
export(pk.calc.mrt.md)
export(pk.calc.ptr)
export(pk.calc.sparse_auc)
export(pk.calc.sparse_auclast)
export(pk.calc.swing)
export(pk.calc.tfirst)
export(pk.calc.thalf.eff)
export(pk.calc.time_above)
export(pk.calc.tlag)
export(pk.calc.tlast)
export(pk.calc.tmax)
export(pk.calc.totdose)
export(pk.calc.vss)
export(pk.calc.vz)
export(pk.nca)
export(pk.nca.interval)
export(pk.tss)
export(pk.tss.monoexponential)
export(pk.tss.stepwise.linear)
export(pknca_units_table)
export(right_join)
export(roundString)
export(roundingSummarize)
export(setDuration)
export(setRoute)
export(set_intervals)
export(signifString)
export(sparse_auc_weight_linear)
export(sparse_mean)
export(superposition)
export(time_calc)
export(ungroup)
export(var_sparse_auc)
importFrom(dplyr,"%>%")
importFrom(dplyr,filter)
importFrom(dplyr,full_join)
importFrom(dplyr,group_by)
importFrom(dplyr,inner_join)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,right_join)
importFrom(dplyr,ungroup)
importFrom(lifecycle,deprecated)
importFrom(nlme,getGroups)
importFrom(rlang,.data)
importFrom(stats,formula)
importFrom(stats,model.frame)
