#' The following functions are defunct
#' @name defunct
#' @param ... Ignored
NULL

#' @describeIn defunct Defunct as of version 0.11
#' @export
check.conc.time <- function(...) {
  .Defunct("assert_conc_time") # nocov
}

# Notes for deprecated items that should be made defunct:
# * interp.extrap.conc arguments interp.method and extrap.method
