% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/exclude.R
\name{normalize_exclude}
\alias{normalize_exclude}
\title{Normalize the exclude column by setting blanks to NA}
\usage{
normalize_exclude(object)
}
\arguments{
\item{object}{The object to extract the exclude column from}
}
\value{
The exclude vector where \code{NA} indicates not to exclude and anything
else indicates to exclude.
}
\description{
Normalize the exclude column by setting blanks to NA
}
