% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/class-PKNCAconc.R, R/class-PKNCAdata.R,
%   R/class-PKNCAdose.R, R/class-PKNCAresults.R
\name{group_vars.PKNCAconc}
\alias{group_vars.PKNCAconc}
\alias{group_vars.PKNCAdata}
\alias{group_vars.PKNCAdose}
\alias{group_vars.PKNCAresults}
\title{Get grouping variables for a PKNCA object}
\usage{
\method{group_vars}{PKNCAconc}(x)

\method{group_vars}{PKNCAdata}(x)

\method{group_vars}{PKNCAdose}(x)

\method{group_vars}{PKNCAresults}(x)
}
\arguments{
\item{x}{The PKNCA object}
}
\value{
A character vector (possibly empty) of the grouping variables
}
\description{
Get grouping variables for a PKNCA object
}
\section{Functions}{
\itemize{
\item \code{group_vars(PKNCAdata)}: Get group_vars for a PKNCAdata object
from the PKNCAconc object within

\item \code{group_vars(PKNCAdose)}: Get group_vars for a PKNCAdose object

\item \code{group_vars(PKNCAresults)}: Get group_vars for a PKNCAresults object
from the PKNCAconc object within

}}
