% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/PKNCA.options.R
\name{PKNCA.choose.option}
\alias{PKNCA.choose.option}
\title{Choose either the value from an option list or the current set value for an
option.}
\usage{
PKNCA.choose.option(name, value = NULL, options = list())
}
\arguments{
\item{name}{The option name requested.}

\item{value}{A value to check for the option (\code{NULL} to choose not to check
the value).}

\item{options}{List of changes to the default PKNCA options (see
\code{PKNCA.options()})}
}
\value{
The value of the option first from the \code{options} list and if it is
not there then from the current settings.
}
\description{
Choose either the value from an option list or the current set value for an
option.
}
\seealso{
Other PKNCA calculation and summary settings: 
\code{\link{PKNCA.options}()},
\code{\link{PKNCA.set.summary}()}
}
\concept{PKNCA calculation and summary settings}
