% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/001-add.interval.col.R
\name{sort.interval.cols}
\alias{sort.interval.cols}
\title{Sort the interval columns by dependencies.}
\usage{
\method{sort}{interval.cols}()
}
\description{
Columns are always to the right of columns that they depend on.
}
