% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pk.calc.simple.R
\name{pk.calc.tlag}
\alias{pk.calc.tlag}
\title{Determine the observed lag time (time before the first
concentration above the limit of quantification or above the first
concentration in the interval)}
\usage{
pk.calc.tlag(conc, time)
}
\arguments{
\item{conc}{Measured concentrations}

\item{time}{Time of the measurement of the concentrations}
}
\value{
The time associated with the first increasing concentration
}
\description{
Determine the observed lag time (time before the first
concentration above the limit of quantification or above the first
concentration in the interval)
}
