% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/defunct.R
\name{defunct}
\alias{defunct}
\alias{check.conc.time}
\title{The following functions are defunct}
\usage{
check.conc.time(...)
}
\arguments{
\item{...}{Ignored}
}
\description{
The following functions are defunct
}
\section{Functions}{
\itemize{
\item \code{check.conc.time()}: Defunct as of version 0.11

}}
