% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tss.monoexponential.R
\name{tss.monoexponential.generate.formula}
\alias{tss.monoexponential.generate.formula}
\title{A helper function to generate the formula and starting values for the
parameters in monoexponential models.}
\usage{
tss.monoexponential.generate.formula(data)
}
\arguments{
\item{data}{The data used for the model}
}
\value{
a list with elements for each of the variables
}
\description{
A helper function to generate the formula and starting values for the
parameters in monoexponential models.
}
