% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dplyr.R
\name{filter.PKNCAresults}
\alias{filter.PKNCAresults}
\alias{filter.PKNCAconc}
\alias{filter.PKNCAdose}
\title{dplyr filtering for PKNCA}
\usage{
\method{filter}{PKNCAresults}(.data, ..., .preserve = FALSE)

\method{filter}{PKNCAconc}(.data, ..., .preserve = FALSE)

\method{filter}{PKNCAdose}(.data, ..., .preserve = FALSE)
}
\arguments{
\item{.data}{A data frame, data frame extension (e.g. a tibble), or a
lazy data frame (e.g. from dbplyr or dtplyr). See \emph{Methods}, below, for
more details.}

\item{...}{<\code{\link[rlang:args_data_masking]{data-masking}}> Expressions that
return a logical value, and are defined in terms of the variables in
\code{.data}. If multiple expressions are included, they are combined with the
\code{&} operator. Only rows for which all conditions evaluate to \code{TRUE} are
kept.}

\item{.preserve}{Relevant when the \code{.data} input is grouped.
If \code{.preserve = FALSE} (the default), the grouping structure
is recalculated based on the resulting data, otherwise the grouping is kept as is.}
}
\description{
dplyr filtering for PKNCA
}
\seealso{
Other dplyr verbs: 
\code{\link{group_by.PKNCAresults}()},
\code{\link{inner_join.PKNCAresults}()},
\code{\link{mutate.PKNCAresults}()}
}
\concept{dplyr verbs}
