% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pk.calc.all.R
\name{any_sparse_dense_in_interval}
\alias{any_sparse_dense_in_interval}
\title{Determine if there are any sparse or dense calculations requested within an interval}
\usage{
any_sparse_dense_in_interval(interval, sparse)
}
\arguments{
\item{interval}{An interval specification}

\item{sparse}{Are the concentration-time data sparse PK (commonly used in
small nonclinical species or with terminal or difficult sampling) or dense
PK (commonly used in clinical studies or larger nonclinical species)?}
}
\value{
A logical value indicating if the interval requests any sparse (if
\code{sparse=TRUE}) or dense (if \code{sparse=FALSE}) calculations.
}
\description{
Determine if there are any sparse or dense calculations requested within an interval
}
\keyword{Internal}
