% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tss.R
\name{pk.tss}
\alias{pk.tss}
\title{Compute the time to steady-state (tss)}
\usage{
pk.tss(..., type = c("monoexponential", "stepwise.linear"), check = TRUE)
}
\arguments{
\item{\dots}{Passed to \code{\link[=pk.tss.monoexponential]{pk.tss.monoexponential()}} or
\code{\link[=pk.tss.stepwise.linear]{pk.tss.stepwise.linear()}}.}

\item{type}{The type of Tss to calculate, either \code{stepwise.linear} or
\code{monoexponential}}

\item{check}{See \code{\link[=pk.tss.data.prep]{pk.tss.data.prep()}}}
}
\value{
A data frame with columns as defined from \code{pk.tss.monoexponential}
and/or \code{pk.tss.stepwise.linear}.
}
\description{
Compute the time to steady-state (tss)
}
\seealso{
Other Time to steady-state calculations: 
\code{\link{pk.tss.monoexponential}()},
\code{\link{pk.tss.stepwise.linear}()}
}
\concept{Time to steady-state calculations}
