% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/impute.R
\name{add_impute_to_intervals}
\alias{add_impute_to_intervals}
\title{Add the imputation column to the intervals, if it is not already there}
\usage{
add_impute_to_intervals(object)
}
\arguments{
\item{object}{The PKNCAdata object to impute data within}
}
\value{
The PKNCAdata object with an impute column added to the intervals (if
it is not already there) and the object$impute set to that column name
}
\description{
Add the imputation column to the intervals, if it is not already there
}
\keyword{internal}
