% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pk.calc.simple.R
\name{pk.calc.kel}
\alias{pk.calc.kel}
\title{Calculate the elimination rate (Kel)}
\usage{
pk.calc.kel(mrt)
}
\arguments{
\item{mrt}{the mean residence time}

\item{kel}{is \code{1/mrt}, not to be confused with lambda.z.}
}
\value{
the numeric value of the elimination rate
}
\description{
Calculate the elimination rate (Kel)
}
