% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pk.calc.simple.R
\name{pk.calc.ptr}
\alias{pk.calc.ptr}
\title{Determine the peak-to-trough ratio}
\usage{
pk.calc.ptr(cmax, ctrough)
}
\arguments{
\item{cmax}{The maximum observed concentration}

\item{ctrough}{The last concentration in an interval}
}
\value{
The ratio of cmax to ctrough (if ctrough == 0, NA)
}
\description{
Determine the peak-to-trough ratio
}
\details{
ptr is \code{cmax/ctrough}.
}
