% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/assertions.R
\name{assert_lambdaz}
\alias{assert_lambdaz}
\title{Assert that a lambda.z value is valid}
\usage{
assert_lambdaz(
  lambda.z,
  any.missing = TRUE,
  .var.name = checkmate::vname(lambda.z)
)
}
\arguments{
\item{lambda.z}{The elimination rate (in units of inverse time) for
extrapolation}

\item{any.missing}{[\code{logical(1)}]\cr
Are vectors with missing values allowed? Default is \code{TRUE}.}

\item{.var.name}{[\code{character(1)}]\cr
Name of the checked object to print in assertions. Defaults to
the heuristic implemented in \code{\link[checkmate]{vname}}.}
}
\value{
\code{lambda.z} or an informative error
}
\description{
Assert that a lambda.z value is valid
}
