% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/class-PKNCAconc.R, R/class-PKNCAdose.R
\name{setDuration.PKNCAconc}
\alias{setDuration.PKNCAconc}
\alias{setDuration}
\alias{setDuration.PKNCAdose}
\title{Set the duration of dosing or measurement}
\usage{
\method{setDuration}{PKNCAconc}(object, duration, ...)

setDuration(object, ...)

\method{setDuration}{PKNCAdose}(object, duration, rate, dose, ...)
}
\arguments{
\item{object}{An object to set a duration on}

\item{duration}{The value to set for the duration or the name of the column
in the data to use for the duration.}

\item{...}{Arguments passed to another setDuration function}

\item{rate}{(for PKNCAdose objects only) The rate of infusion}

\item{dose}{(for PKNCAdose objects only) The dose amount}
}
\value{
The object with duration set
}
\description{
Set the duration of dosing or measurement
}
