% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/class-PKNCAdata.R
\name{PKNCAdata}
\alias{PKNCAdata}
\alias{PKNCAdata.PKNCAconc}
\alias{PKNCAdata.PKNCAdose}
\alias{PKNCAdata.default}
\title{Create a PKNCAdata object.}
\usage{
PKNCAdata(data.conc, data.dose, ...)

\method{PKNCAdata}{PKNCAconc}(data.conc, data.dose, ...)

\method{PKNCAdata}{PKNCAdose}(data.conc, data.dose, ...)

\method{PKNCAdata}{default}(
  data.conc,
  data.dose,
  ...,
  formula.conc,
  formula.dose,
  impute = NA_character_,
  intervals,
  units,
  options = list()
)
}
\arguments{
\item{data.conc}{Concentration data as a \code{PKNCAconc} object or a data frame}

\item{data.dose}{Dosing data as a \code{PKNCAdose} object (see details)}

\item{...}{arguments passed to \code{PKNCAdata.default}}

\item{formula.conc}{Formula for making a \code{PKNCAconc} object with \code{data.conc}.
This must be given if \code{data.conc} is a data.frame, and it must not be given
if \code{data.conc} is a \code{PKNCAconc} object.}

\item{formula.dose}{Formula for making a \code{PKNCAdose} object with \code{data.dose}.
This must be given if \code{data.dose} is a data.frame, and it must not be given
if \code{data.dose} is a \code{PKNCAdose} object.}

\item{impute}{Methods for imputation.  \code{NA} for to search for the column
named "impute" in the intervals or no imputation if that column does not
exist, a comma-or space-separated list of names, or the name of a column in
the \code{intervals} data.frame.  See
\code{vignette("v08-data-imputation", package="PKNCA")} for more details.}

\item{intervals}{A data frame with the AUC interval specifications as defined
in \code{\link[=check.interval.specification]{check.interval.specification()}}.  If missing, this will be
automatically chosen by \code{\link[=choose.auc.intervals]{choose.auc.intervals()}}. (see details)}

\item{units}{A data.frame of unit assignments and conversions as created by
\code{\link[=pknca_units_table]{pknca_units_table()}}}

\item{options}{List of changes to the default PKNCA options (see
\code{PKNCA.options()})}
}
\value{
A PKNCAdata object with concentration, dose, interval, and
calculation options stored (note that PKNCAdata objects can also have
results after a NCA calculations are done to the data).
}
\description{
\code{PKNCAdata()} combines \code{PKNCAconc} and \code{PKNCAdose} objects and adds in the
intervals for PK calculations.
}
\details{
If \code{data.dose} is not given or is \code{NA}, then the \code{intervals} must be
given.  At least one of \code{data.dose} and \code{intervals} must be given.
}
\seealso{
\code{\link[=choose.auc.intervals]{choose.auc.intervals()}}, \code{\link[=pk.nca]{pk.nca()}}, \code{\link[=pknca_units_table]{pknca_units_table()}}

Other PKNCA objects: 
\code{\link{PKNCAconc}()},
\code{\link{PKNCAdose}()},
\code{\link{PKNCAresults}()}
}
\concept{PKNCA objects}
