% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/class-PKNCAconc.R, R/class-PKNCAdata.R,
%   R/class-PKNCAresults.R, R/sparse.R
\name{is_sparse_pk.PKNCAconc}
\alias{is_sparse_pk.PKNCAconc}
\alias{is_sparse_pk.PKNCAdata}
\alias{is_sparse_pk.PKNCAresults}
\alias{is_sparse_pk}
\title{Is a PKNCA object used for sparse PK?}
\usage{
\method{is_sparse_pk}{PKNCAconc}(object)

\method{is_sparse_pk}{PKNCAdata}(object)

\method{is_sparse_pk}{PKNCAresults}(object)

is_sparse_pk(object)
}
\arguments{
\item{object}{The object to see if it includes sparse PK}
}
\value{
\code{TRUE} if sparse and \code{FALSE} if dense (not sparse)
}
\description{
Is a PKNCA object used for sparse PK?
}
