% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/assertions.R
\name{assert_dosetau}
\alias{assert_dosetau}
\title{Assert that a value is a dosing interval}
\usage{
assert_dosetau(tau)
}
\arguments{
\item{tau}{The dosing interval}
}
\value{
\code{tau} or an informative error
}
\description{
Assert that a value is a dosing interval
}
