% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/class-PKNCAconc.R, R/class-PKNCAdose.R
\name{model.frame.PKNCAconc}
\alias{model.frame.PKNCAconc}
\alias{model.frame.PKNCAdose}
\title{Extract the columns used in the formula (in order) from a PKNCAconc
or PKNCAdose object.}
\usage{
\method{model.frame}{PKNCAconc}(formula, ...)

\method{model.frame}{PKNCAdose}(formula, ...)
}
\arguments{
\item{formula}{The object to use (parameter name is \code{formula} to use the
generic function)}

\item{\dots}{Unused}
}
\value{
A data frame with the columns from the object in formula order.
}
\description{
Extract the columns used in the formula (in order) from a PKNCAconc
or PKNCAdose object.
}
