% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/class-summary_PKNCAresults.R
\name{print.summary_PKNCAresults}
\alias{print.summary_PKNCAresults}
\title{Print the results summary}
\usage{
\method{print}{summary_PKNCAresults}(x, ...)
}
\arguments{
\item{x}{A summary_PKNCAresults object}

\item{...}{passed to print.data.frame (\code{row.names} is always set to \code{FALSE})}
}
\value{
\code{x} invisibly
}
\description{
Print the results summary
}
\seealso{
\code{\link[=summary.PKNCAresults]{summary.PKNCAresults()}}
}
