% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/unit-support.R
\name{pknca_units_add_paren}
\alias{pknca_units_add_paren}
\title{Add parentheses to a unit value, if needed}
\usage{
pknca_units_add_paren(unit)
}
\arguments{
\item{unit}{The text of the unit}
}
\value{
The unit with parentheses around it, if needed
}
\description{
Add parentheses to a unit value, if needed
}
\keyword{Internal}
