% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/class-PKNCAconc.R, R/class-PKNCAdose.R,
%   R/class-PKNCAresults.R, R/class-general.R
\name{getDataName.PKNCAconc}
\alias{getDataName.PKNCAconc}
\alias{getDataName.PKNCAdose}
\alias{getDataName.PKNCAresults}
\alias{getDataName}
\alias{getDataName.default}
\title{Get the name of the element containing the data for the current
object.}
\usage{
\method{getDataName}{PKNCAconc}(object)

\method{getDataName}{PKNCAdose}(object)

\method{getDataName}{PKNCAresults}(object)

getDataName(object)

\method{getDataName}{default}(object)
}
\arguments{
\item{object}{The object to get the data name from.}
}
\value{
A character scalar with the name of the data object (or \code{NULL} if the
method does not apply).
}
\description{
Get the name of the element containing the data for the current
object.
}
\section{Methods (by class)}{
\itemize{
\item \code{getDataName(default)}: If no data name exists, returns NULL.

}}
\seealso{
Other PKNCA object extractors: 
\code{\link{getDepVar}()},
\code{\link{getIndepVar}()}
}
\concept{PKNCA object extractors}
\keyword{Internal}
