% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/PKNCA.options.R
\name{PKNCA.options.describe}
\alias{PKNCA.options.describe}
\title{Describe a PKNCA.options option by name.}
\usage{
PKNCA.options.describe(name)
}
\arguments{
\item{name}{The option name requested.}
}
\value{
A character string of the description.
}
\description{
Describe a PKNCA.options option by name.
}
\seealso{
\code{\link[=PKNCA.options]{PKNCA.options()}}
}
