% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/provenance.R
\name{addProvenance}
\alias{addProvenance}
\title{Add a hash and associated information to enable checking object provenance.}
\usage{
addProvenance(object, replace = FALSE)
}
\arguments{
\item{object}{The object to add provenance}

\item{replace}{Replace provenance if the object already has a provenance
attribute.  (If the object already has provenance and \code{replace} is \code{FALSE},
then an error will be raised.)}
}
\value{
The object with provenance as an added item
}
\description{
Add a hash and associated information to enable checking object provenance.
}
\seealso{
\code{\link[=checkProvenance]{checkProvenance()}}
}
