% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/general.functions.R
\name{check.conversion}
\alias{check.conversion}
\title{Check that the conversion to a data type does not change the number
of NA values}
\usage{
check.conversion(x, FUN, ...)
}
\arguments{
\item{x}{the value to convert}

\item{FUN}{the function to use for conversion}

\item{\dots}{arguments passed to \code{FUN}}
}
\value{
\code{FUN(x, ...)} or an error if the set of NAs change.
}
\description{
Check that the conversion to a data type does not change the number
of NA values
}
