% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/impute.R
\name{PKNCA_impute_fun_list}
\alias{PKNCA_impute_fun_list}
\title{Separate out a vector of PKNCA imputation methods into a list of functions}
\usage{
PKNCA_impute_fun_list(x)
}
\arguments{
\item{x}{The character vector of PKNCA imputation method functions (without
the \code{PKNCA_impute_method_} part)}
}
\value{
A list of character vectors of functions to run.
}
\description{
An error will be raised if the functions are not found.
}
\details{
This function is not for use by users of PKNCA.
}
\keyword{Internal}
