% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pk.calc.simple.R
\name{pk.calc.ceoi}
\alias{pk.calc.ceoi}
\title{Determine the concentration at the end of infusion}
\usage{
pk.calc.ceoi(conc, time, duration.dose = NA, check = TRUE)
}
\arguments{
\item{conc}{Measured concentrations}

\item{time}{Time of the measurement of the concentrations}

\item{duration.dose}{The duration for the dosing administration (typically
from IV infusion)}

\item{check}{Run \code{\link[=assert_conc_time]{assert_conc_time()}}?}
}
\value{
The concentration at the end of the infusion, \code{NA} if
\code{duration.dose} is \code{NA}, or \code{NA} if all \code{time != duration.dose}
}
\description{
Determine the concentration at the end of infusion
}
