% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/class-general.R
\name{getDepVar}
\alias{getDepVar}
\title{Get the dependent variable (left hand side of the formula) from a
PKNCA object.}
\usage{
getDepVar(x, ...)
}
\arguments{
\item{x}{The object to extract the formula from}

\item{\dots}{Unused}
}
\value{
The vector of the dependent variable from the object.
}
\description{
Get the dependent variable (left hand side of the formula) from a
PKNCA object.
}
\seealso{
Other PKNCA object extractors: 
\code{\link{getDataName.PKNCAconc}()},
\code{\link{getIndepVar}()}
}
\concept{PKNCA object extractors}
