% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/check.intervals.R
\name{get.parameter.deps}
\alias{get.parameter.deps}
\title{Get all columns that depend on a parameter}
\usage{
get.parameter.deps(x)
}
\arguments{
\item{x}{The parameter name (as a character string)}
}
\value{
A character vector of parameter names that depend on the parameter
\code{x}.  If none depend on \code{x}, then the result will be an empty vector.
}
\description{
Get all columns that depend on a parameter
}
\seealso{
Other Interval specifications: 
\code{\link{add.interval.col}()},
\code{\link{check.interval.deps}()},
\code{\link{check.interval.specification}()},
\code{\link{choose.auc.intervals}()},
\code{\link{get.interval.cols}()}
}
\concept{Interval specifications}
