% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pk.calc.simple.R
\name{pk.calc.swing}
\alias{pk.calc.swing}
\title{Determine the PK swing}
\usage{
pk.calc.swing(cmax, cmin)
}
\arguments{
\item{cmax}{The maximum observed concentration}

\item{cmin}{The minimum observed concentration}
}
\value{
The swing above the minimum concentration.  If \code{cmin} is zero, then
the result is infinity.
}
\description{
Determine the PK swing
}
\details{
swing is \code{100*(cmax - cmin)/cmin}.
}
