% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pk.calc.simple.R
\name{pk.calc.totdose}
\alias{pk.calc.totdose}
\title{Extract the dose used for calculations}
\usage{
pk.calc.totdose(dose)
}
\arguments{
\item{dose}{the dose administered}
}
\value{
The total dose for an interval
}
\description{
Extract the dose used for calculations
}
