% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pk.calc.simple.R
\name{pk.calc.tlast}
\alias{pk.calc.tlast}
\alias{pk.calc.tfirst}
\title{Determine time of last observed concentration above the limit of
quantification.}
\usage{
pk.calc.tlast(conc, time, check = TRUE)

pk.calc.tfirst(conc, time, check = TRUE)
}
\arguments{
\item{conc}{Measured concentrations}

\item{time}{Time of the measurement of the concentrations}

\item{check}{Run \code{\link[=assert_conc_time]{assert_conc_time()}}?}
}
\value{
The time of the last observed concentration measurement
}
\description{
\code{NA} will be returned if all \code{conc} are \code{NA} or 0.
}
\section{Functions}{
\itemize{
\item \code{pk.calc.tfirst()}: Determine the first concentration above
the limit of quantification.

}}
