% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sparse.R
\name{as_sparse_pk}
\alias{as_sparse_pk}
\title{Generate a sparse_pk object}
\usage{
as_sparse_pk(conc, time, subject)
}
\arguments{
\item{conc}{Measured concentrations}

\item{time}{Time of the measurement of the concentrations}

\item{subject}{Subject identifiers (may be any class; may not be null)}
}
\value{
A sparse_pk object which is a list of lists.  The inner lists have
elements named: "time", The time of measurement; "conc", The concentration
measured; "subject", The subject identifiers.  The object will usually be
modified by future functions to add more named elements to the inner list.
}
\description{
Generate a sparse_pk object
}
\seealso{
Other Sparse Methods: 
\code{\link{pk.calc.sparse_auc}()},
\code{\link{sparse_auc_weight_linear}()},
\code{\link{sparse_mean}()}
}
\concept{Sparse Methods}
