% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as_various.R
\name{as_PKNCAconc}
\alias{as_PKNCAconc}
\alias{as_PKNCAdose}
\alias{as_PKNCAdata}
\alias{as_PKNCAresults}
\title{Convert an object into a PKNCAconc object}
\usage{
as_PKNCAconc(x, ...)

as_PKNCAdose(x, ...)

as_PKNCAdata(x, ...)

as_PKNCAresults(x, ...)
}
\arguments{
\item{x}{The object to convert}

\item{...}{Passed to subsequent methods}
}
\value{
A converted object
}
\description{
Convert an object into a PKNCAconc object
}
\section{Functions}{
\itemize{
\item \code{as_PKNCAdose()}: Convert an object into a PKNCAdose object

\item \code{as_PKNCAdata()}: Convert an object into a PKNCAdata object

\item \code{as_PKNCAresults()}: Convert an object into a PKNCAresults object

}}
