% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/check.intervals.R
\name{check.interval.deps}
\alias{check.interval.deps}
\title{Take in a single row of an interval specification and return that row updated
with any additional calculations that must be done to fulfill all
dependencies.}
\usage{
check.interval.deps(x)
}
\arguments{
\item{x}{A data frame with one or more rows of the PKNCA interval}
}
\value{
The interval specification with additional calculations added where
requested outputs require them.
}
\description{
Take in a single row of an interval specification and return that row updated
with any additional calculations that must be done to fulfill all
dependencies.
}
\seealso{
Other Interval specifications: 
\code{\link{add.interval.col}()},
\code{\link{check.interval.specification}()},
\code{\link{choose.auc.intervals}()},
\code{\link{get.interval.cols}()},
\code{\link{get.parameter.deps}()}
}
\concept{Interval specifications}
