% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pk.calc.simple.R
\name{pk.calc.deg.fluc}
\alias{pk.calc.deg.fluc}
\title{Determine the degree of fluctuation}
\usage{
pk.calc.deg.fluc(cmax, cmin, cav)
}
\arguments{
\item{cmax}{The maximum observed concentration}

\item{cmin}{The minimum observed concentration}

\item{cav}{The average concentration in the interval}
}
\value{
The degree of fluctuation around the average concentration.
}
\description{
Determine the degree of fluctuation
}
\details{
deg.fluc is \code{100*(cmax - cmin)/cav}.
}
