% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sparse.R
\name{sparse_pk_attribute}
\alias{sparse_pk_attribute}
\title{Set or get a sparse_pk object attribute}
\usage{
sparse_pk_attribute(sparse_pk, ...)
}
\arguments{
\item{sparse_pk}{A sparse_pk object from \code{\link[=as_sparse_pk]{as_sparse_pk()}}}

\item{...}{Either a character string (to get that value) or a named vector
the same length as \code{sparse_pk} to set the value.}
}
\value{
Either the attribute value or an updated \code{sparse_pk} object
}
\description{
Set or get a sparse_pk object attribute
}
\keyword{Internal}
