% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dplyr.R
\name{mutate.PKNCAresults}
\alias{mutate.PKNCAresults}
\alias{mutate.PKNCAconc}
\alias{mutate.PKNCAdose}
\title{dplyr mutate-based modification for PKNCA}
\usage{
\method{mutate}{PKNCAresults}(.data, ...)

\method{mutate}{PKNCAconc}(.data, ...)

\method{mutate}{PKNCAdose}(.data, ...)
}
\arguments{
\item{.data}{A data frame, data frame extension (e.g. a tibble), or a
lazy data frame (e.g. from dbplyr or dtplyr). See \emph{Methods}, below, for
more details.}

\item{...}{<\code{\link[rlang:args_data_masking]{data-masking}}> Name-value pairs.
The name gives the name of the column in the output.

The value can be:
\itemize{
\item A vector of length 1, which will be recycled to the correct length.
\item A vector the same length as the current group (or the whole data frame
if ungrouped).
\item \code{NULL}, to remove the column.
\item A data frame or tibble, to create multiple columns in the output.
}}
}
\description{
dplyr mutate-based modification for PKNCA
}
\seealso{
Other dplyr verbs: 
\code{\link{filter.PKNCAresults}()},
\code{\link{group_by.PKNCAresults}()},
\code{\link{inner_join.PKNCAresults}()}
}
\concept{dplyr verbs}
