% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/unit-support.R
\name{pknca_find_units_param}
\alias{pknca_find_units_param}
\title{Find NCA parameters with a given unit type}
\usage{
pknca_find_units_param(unit_type)
}
\arguments{
\item{unit_type}{The type of unit as assigned with \code{add.interval.col}}
}
\value{
A character vector of parameters with a given unit type
}
\description{
Find NCA parameters with a given unit type
}
\keyword{Internal}
