% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/001-add.interval.col.R
\name{get.interval.cols}
\alias{get.interval.cols}
\title{Get the columns that can be used in an interval specification}
\usage{
get.interval.cols()
}
\value{
A list with named elements for each parameter.  Each list element
contains the parameter definition.
}
\description{
Get the columns that can be used in an interval specification
}
\examples{
get.interval.cols()
}
\seealso{
\code{\link[=check.interval.specification]{check.interval.specification()}} and the vignette "Selection of
Calculation Intervals"

Other Interval specifications: 
\code{\link{add.interval.col}()},
\code{\link{check.interval.deps}()},
\code{\link{check.interval.specification}()},
\code{\link{choose.auc.intervals}()},
\code{\link{get.parameter.deps}()}
}
\concept{Interval specifications}
