% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sparse.R
\name{sparse_to_dense_pk}
\alias{sparse_to_dense_pk}
\title{Extract the mean concentration-time profile as a data.frame}
\usage{
sparse_to_dense_pk(sparse_pk)
}
\arguments{
\item{sparse_pk}{A sparse_pk object from \code{\link[=as_sparse_pk]{as_sparse_pk()}}}
}
\value{
A data.frame with names of "conc" and "time"
}
\description{
Extract the mean concentration-time profile as a data.frame
}
\keyword{Internal}
