% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AIC.list.R
\name{get.first.model}
\alias{get.first.model}
\title{Get the first model from a list of models}
\usage{
get.first.model(object)
}
\arguments{
\item{object}{the list of (lists of, ...) models}
}
\value{
The first item in the \code{object} that is not a list or \code{NA}.  If \code{NA}
is passed in or the list (of lists) is all \code{NA}, then \code{NA} is returned.
}
\description{
Get the first model from a list of models
}
