% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/auc_integrate.R
\name{interp_extrap_conc_method}
\alias{interp_extrap_conc_method}
\alias{interpolate_conc_linear}
\alias{interpolate_conc_log}
\alias{extrapolate_conc_lambdaz}
\title{Interpolate or extrapolate concentrations using the provided method}
\usage{
interpolate_conc_linear(conc_1, conc_2, time_1, time_2, time_out)

interpolate_conc_log(conc_1, conc_2, time_1, time_2, time_out)

extrapolate_conc_lambdaz(clast, lambda.z, tlast, time_out)
}
\arguments{
\item{conc_1, conc_2}{The concentration at time1 and time2}

\item{time_1, time_2}{The time value associated with conc1 and conc2}

\item{time_out}{Time when interpolation is requested}

\item{clast}{The concentration at the last time above the lower LOQ}

\item{lambda.z}{The elimination rate (in units of inverse time) for
extrapolation}

\item{tlast}{The time of the last concentration above the lower limit of
quantification (LOQ)}
}
\value{
The interpolated or extrapolated value using the correct method
}
\description{
Interpolate or extrapolate concentrations using the provided method
}
\keyword{Internal}
