% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pk.calc.simple.R
\name{adj.r.squared}
\alias{adj.r.squared}
\title{Calculate the adjusted r-squared value}
\usage{
adj.r.squared(r.sq, n)
}
\arguments{
\item{r.sq}{The r-squared value}

\item{n}{The number of points}
}
\value{
The numeric adjusted r-squared value
}
\description{
Calculate the adjusted r-squared value
}
