% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parse_formula_to_cols.R
\name{findOperator}
\alias{findOperator}
\title{Find the first occurrence of an operator in a formula and return
the left, right, or both sides of the operator.}
\usage{
findOperator(x, op, side)
}
\arguments{
\item{x}{The formula to parse}

\item{op}{The operator to search for (e.g. \code{+}, \code{-}, \code{*}, \code{/}, ...)}

\item{side}{Which side of the operator would you like to see: 'left',
'right', or 'both'.}
}
\value{
The side of the operator requested, NA if requesting the left side
of a unary operator, and NULL if the operator is not found.
}
\description{
Find the first occurrence of an operator in a formula and return
the left, right, or both sides of the operator.
}
\seealso{
Other Formula parsing: 
\code{\link{parse_formula_to_cols}()}
}
\concept{Formula parsing}
