% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pk.calc.simple.R
\name{pk.calc.vz}
\alias{pk.calc.vz}
\title{Calculate the terminal volume of distribution (Vz)}
\usage{
pk.calc.vz(cl, lambda.z)
}
\arguments{
\item{cl}{the clearance (or apparent observed clearance)}

\item{lambda.z}{The elimination rate (in units of inverse time) for
extrapolation}
}
\description{
Calculate the terminal volume of distribution (Vz)
}
\details{
vz is \code{cl/lambda.z}.
}
