% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pk.calc.all.R
\name{pk_nca_result_to_df}
\alias{pk_nca_result_to_df}
\title{Convert the grouping info and list of results for each group into a results
data.frame}
\usage{
pk_nca_result_to_df(group_info, result)
}
\arguments{
\item{group_info}{A data.frame of grouping columns}

\item{result}{A list of data.frames with the results from NCA parameter
calculations}
}
\value{
A data.frame with group_info and result combined, warnings filtered
out, and results unnested.
}
\description{
Convert the grouping info and list of results for each group into a results
data.frame
}
\keyword{Internal}
