% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/class-PKNCAconc.R, R/class-PKNCAdose.R
\name{formula.PKNCAconc}
\alias{formula.PKNCAconc}
\alias{formula.PKNCAdose}
\title{Extract the formula from a PKNCAconc object.}
\usage{
\method{formula}{PKNCAconc}(x, ...)

\method{formula}{PKNCAdose}(x, ...)
}
\arguments{
\item{x}{The object to extract the formula from.}

\item{\dots}{Unused}
}
\value{
A formula object
}
\description{
Extract the formula from a PKNCAconc object.
}
