% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/class-summary_PKNCAresults.R
\name{roundingSummarize}
\alias{roundingSummarize}
\title{During the summarization of PKNCAresults, do the rounding of values
based on the instructions given.}
\usage{
roundingSummarize(x, name)
}
\arguments{
\item{x}{The values to summarize}

\item{name}{The NCA parameter name (matching a parameter name in
\code{\link[=PKNCA.set.summary]{PKNCA.set.summary()}})}
}
\value{
A string of the rounded value
}
\description{
During the summarization of PKNCAresults, do the rounding of values
based on the instructions given.
}
