% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/exclude.R
\name{setExcludeColumn}
\alias{setExcludeColumn}
\title{Set the exclude parameter on an object}
\usage{
setExcludeColumn(object, exclude = NULL, dataname = "data")
}
\arguments{
\item{object}{The object to set the exclude column on.}

\item{exclude}{The column name to set as the exclude value.}

\item{dataname}{The name of the data.frame within the object to add the
exclude column to.}
}
\value{
The object with an exclude column and attribute
}
\description{
This function adds the exclude column to an object.  To change the
exclude value, use the \code{\link[=exclude]{exclude()}} function.
}
