% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/exclude_nca.R
\name{exclude_nca}
\alias{exclude_nca}
\alias{exclude_nca_span.ratio}
\alias{exclude_nca_max.aucinf.pext}
\alias{exclude_nca_count_conc_measured}
\alias{exclude_nca_min.hl.r.squared}
\alias{exclude_nca_min.hl.adj.r.squared}
\alias{exclude_nca_tmax_early}
\alias{exclude_nca_tmax_0}
\title{Exclude NCA parameters based on examining the parameter set.}
\usage{
exclude_nca_span.ratio(min.span.ratio)

exclude_nca_max.aucinf.pext(max.aucinf.pext)

exclude_nca_count_conc_measured(
  min_count,
  exclude_param_pattern = c("^aucall", "^aucinf", "^aucint", "^auciv", "^auclast",
    "^aumc", "^sparse_auc")
)

exclude_nca_min.hl.r.squared(min.hl.r.squared)

exclude_nca_min.hl.adj.r.squared(min.hl.adj.r.squared = 0.9)

exclude_nca_tmax_early(tmax_early = 0)

exclude_nca_tmax_0()
}
\arguments{
\item{min.span.ratio}{The minimum acceptable span ratio (uses
\code{PKNCA.options("min.span.ratio")} if not provided).}

\item{max.aucinf.pext}{The maximum acceptable percent AUC extrapolation (uses
\code{PKNCA.options("max.aucinf.pext")} if not provided).}

\item{min_count}{Minimum number of measured concentrations}

\item{exclude_param_pattern}{Character vector of regular expression patterns to exclude}

\item{min.hl.r.squared}{The minimum acceptable r-squared value for half-life
(uses \code{PKNCA.options("min.hl.r.squared")} if not provided).}

\item{min.hl.adj.r.squared}{The minimum acceptable adjusted r-squared for half-life
(uses 0.9 if not provided).}

\item{tmax_early}{The time for Tmax which is considered too early to be a valid NCA result}
}
\description{
Exclude NCA parameters based on examining the parameter set.
}
\section{Functions}{
\itemize{
\item \code{exclude_nca_span.ratio()}: Exclude based on span.ratio

\item \code{exclude_nca_max.aucinf.pext()}: Exclude based on AUC percent extrapolated (both observed and predicted)

\item \code{exclude_nca_count_conc_measured()}: Exclude AUC measurements based on count of concentrations measured and not below the lower limit of quantification

\item \code{exclude_nca_min.hl.r.squared()}: Exclude based on half-life r-squared

\item \code{exclude_nca_min.hl.adj.r.squared()}: Exclude based on half-life adjusted r-squared

\item \code{exclude_nca_tmax_early()}: Exclude based on implausibly early Tmax (often used for extravascular dosing with a Tmax value of 0)

\item \code{exclude_nca_tmax_0()}: Exclude based on implausibly early Tmax (special case for \code{tmax_early = 0})

}}
\examples{
my_conc <- PKNCAconc(data.frame(conc=1.1^(3:0),
                                time=0:3,
                                subject=1),
                     conc~time|subject)
my_data <- PKNCAdata(my_conc,
                     intervals=data.frame(start=0, end=Inf,
                                          aucinf.obs=TRUE,
                                          aucpext.obs=TRUE))
my_result <- pk.nca(my_data)
my_result_excluded <- exclude(my_result,
                              FUN=exclude_nca_max.aucinf.pext())
as.data.frame(my_result_excluded)
}
\seealso{
Other Result exclusions: 
\code{\link{exclude}()}
}
\concept{Result exclusions}
