% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/set_and_assert_intervals.R
\name{set_intervals}
\alias{set_intervals}
\title{Set Intervals}
\usage{
set_intervals(data, intervals)
}
\arguments{
\item{data}{PKNCAdata object}

\item{intervals}{Proposed intervals}
}
\value{
The data object with the intervals set.
}
\description{
Takes in two objects, the PKNCAdata object and the proposed intervals.
It will then check that the intervals are valid, given the data object.
If the intervals are valid, it will set them in the object.
It will return the data object with the intervals set.
}
