% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/provenance.R
\name{print.provenance}
\alias{print.provenance}
\title{Print the summary of a provenance object}
\usage{
\method{print}{provenance}(x, ...)
}
\arguments{
\item{x}{The object to be printed}

\item{...}{Ignored}
}
\value{
invisible text of the printed information
}
\description{
Print the summary of a provenance object
}
