% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/class-PKNCAdata.R
\name{print.PKNCAdata}
\alias{print.PKNCAdata}
\title{Print a PKNCAdata object}
\usage{
\method{print}{PKNCAdata}(x, ...)
}
\arguments{
\item{x}{The object to print}

\item{...}{Arguments passed on to \code{\link[=print.PKNCAconc]{print.PKNCAconc()}} and \code{\link[=print.PKNCAdose]{print.PKNCAdose()}}}
}
\description{
Print a PKNCAdata object
}
