% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/assertions.R
\name{assert_number_between}
\alias{assert_number_between}
\title{Confirm that a value is greater than another value}
\usage{
assert_number_between(
  x,
  ...,
  na.ok = FALSE,
  len = 1,
  .var.name = checkmate::vname(x)
)
}
\arguments{
\item{x}{[any]\cr
Object to check.}

\item{...}{Passed to \code{assert_numeric_between()}}

\item{na.ok}{[\code{logical(1)}]\cr
Are missing values allowed? Default is \code{FALSE}.}

\item{len}{Ignored (must be 1)}

\item{.var.name}{[\code{character(1)}]\cr
Name of the checked object to print in assertions. Defaults to
the heuristic implemented in \code{\link[checkmate]{vname}}.}
}
\value{
\code{x} or an informative error
}
\description{
Confirm that a value is greater than another value
}
