% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/assertions.R
\name{assert_aucmethod}
\alias{assert_aucmethod}
\title{Assert that a value is a valid AUC method}
\usage{
assert_aucmethod(method = c("lin up/log down", "linear", "lin-log"))
}
\arguments{
\item{method}{The method for integration (one of 'lin up/log down',
'lin-log', or 'linear')}
}
\value{
\code{method} or an informative error
}
\description{
Assert that a value is a valid AUC method
}
