% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/assertions.R
\name{assert_PKNCAdata}
\alias{assert_PKNCAdata}
\title{Assert that an object is a PKNCAdata object}
\usage{
assert_PKNCAdata(object)
}
\arguments{
\item{object}{The PKNCAdata object}
}
\value{
The PKNCAdata object (confirmed to be usable)
}
\description{
Assert that an object is a PKNCAdata object
}
