% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/class-PKNCAconc.R, R/class-PKNCAdose.R
\name{print.PKNCAconc}
\alias{print.PKNCAconc}
\alias{summary.PKNCAconc}
\alias{print.PKNCAdose}
\alias{summary.PKNCAdose}
\title{Print and/or summarize a PKNCAconc or PKNCAdose object.}
\usage{
\method{print}{PKNCAconc}(x, n = 6, summarize = FALSE, ...)

\method{summary}{PKNCAconc}(object, n = 0, summarize = TRUE, ...)

\method{print}{PKNCAdose}(x, n = 6, summarize = FALSE, ...)

\method{summary}{PKNCAdose}(object, n = 0, summarize = TRUE, ...)
}
\arguments{
\item{x}{The object to print}

\item{n}{The number of rows of data to show (see
\code{\link[=head]{head()}})}

\item{summarize}{Summarize the nested number of groups}

\item{\dots}{Arguments passed to \code{print.formula} and
\code{print.data.frame}}

\item{object}{The object to summarize}
}
\description{
Print and/or summarize a PKNCAconc or PKNCAdose object.
}
