% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/zzz-pk.calc.dn.R
\name{pk.calc.dn}
\alias{pk.calc.dn}
\title{Determine dose normalized NCA parameter}
\usage{
pk.calc.dn(parameter, dose)
}
\arguments{
\item{parameter}{Parameter to dose normalize}

\item{dose}{Dose in units compatible with the area under the curve}
}
\value{
a number for dose normalized AUC
}
\description{
Determine dose normalized NCA parameter
}
\examples{
pk.calc.dn(90, 10)
}
