% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pk.calc.all.R
\name{pk.nca.interval}
\alias{pk.nca.interval}
\title{Compute all PK parameters for a single concentration-time data set}
\usage{
pk.nca.interval(
  conc,
  time,
  volume,
  duration.conc,
  dose,
  time.dose,
  duration.dose,
  route,
  conc.group = NULL,
  time.group = NULL,
  volume.group = NULL,
  duration.conc.group = NULL,
  dose.group = NULL,
  time.dose.group = NULL,
  duration.dose.group = NULL,
  route.group = NULL,
  impute_method = NA_character_,
  include_half.life = NULL,
  exclude_half.life = NULL,
  subject,
  sparse,
  interval,
  options = list()
)
}
\arguments{
\item{conc}{Measured concentrations}

\item{time}{Time of the measurement of the concentrations}

\item{volume, volume.group}{The volume (or mass) of the concentration
measurement for the current interval or all data for the group (typically
for urine and fecal measurements)}

\item{duration.conc, duration.conc.group}{The duration of the concentration
measurement for the current interval or all data for the group (typically
for urine and fecal measurements)}

\item{dose, dose.group}{Dose amount (may be a scalar or vector) for the
current interval or all data for the group}

\item{time.dose}{Time of the dose for the current interval (must be the same
length as \code{dose})}

\item{duration.dose}{The duration of the dose administration for the current
interval (typically zero for extravascular and intravascular bolus and
nonzero for intravascular infusion)}

\item{route, route.group}{The route of dosing for the current interval or all
data for the group}

\item{conc.group}{All concentrations measured for the group}

\item{time.group}{Time of all concentrations measured for the group}

\item{time.dose.group}{Time of the dose for all data for the group (must be
the same length as \code{dose.group})}

\item{duration.dose.group}{The duration of the dose administration for all
data for the group (typically zero for extravascular and intravascular
bolus and nonzero for intravascular infusion)}

\item{impute_method}{The method to use for imputation as a character string}

\item{include_half.life}{An optional boolean vector of the concentration
measurements to include in the half-life calculation. If given, no
half-life point selection will occur.}

\item{exclude_half.life}{An optional boolean vector of the concentration
measurements to exclude from the half-life calculation.}

\item{subject}{Subject identifiers (used for sparse calculations)}

\item{sparse}{Should only sparse calculations be performed (TRUE) or only
dense calculations (FALSE)?}

\item{interval}{One row of an interval definition (see
\code{\link[=check.interval.specification]{check.interval.specification()}} for how to define the interval.}

\item{options}{List of changes to the default PKNCA options (see
\code{PKNCA.options()})}
}
\value{
A data frame with the start and end time along with all PK
parameters for the \code{interval}
}
\description{
For one subject/time range, compute all available PK parameters. All the
internal options should be set by \code{\link[=PKNCA.options]{PKNCA.options()}} prior to running.  The
only part that changes with a call to this function is the \code{conc}entration
and \code{time}.
}
\seealso{
\code{\link[=check.interval.specification]{check.interval.specification()}}
}
