% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/class-PKNCAdata.R
\name{summary.PKNCAdata}
\alias{summary.PKNCAdata}
\title{Summarize a PKNCAdata object showing important details about the
concentration, dosing, and interval information.}
\usage{
\method{summary}{PKNCAdata}(object, ...)
}
\arguments{
\item{object}{The PKNCAdata object to summarize.}

\item{...}{arguments passed on to \code{\link[=print.PKNCAdata]{print.PKNCAdata()}}}
}
\description{
Summarize a PKNCAdata object showing important details about the
concentration, dosing, and interval information.
}
