% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/class-general.R, R/dplyr.R
\docType{import}
\name{reexports}
\alias{reexports}
\alias{getGroups}
\alias{inner_join}
\alias{left_join}
\alias{right_join}
\alias{full_join}
\alias{filter}
\alias{group_by}
\alias{ungroup}
\alias{mutate}
\title{Objects exported from other packages}
\keyword{internal}
\description{
These objects are imported from other packages. Follow the links
below to see their documentation.

\describe{
  \item{dplyr}{\code{\link[dplyr]{filter}}, \code{\link[dplyr:mutate-joins]{full_join}}, \code{\link[dplyr]{group_by}}, \code{\link[dplyr:mutate-joins]{inner_join}}, \code{\link[dplyr:mutate-joins]{left_join}}, \code{\link[dplyr]{mutate}}, \code{\link[dplyr:mutate-joins]{right_join}}, \code{\link[dplyr:group_by]{ungroup}}}

  \item{nlme}{\code{\link[nlme]{getGroups}}}
}}

