% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/impute.R
\name{get_impute_method}
\alias{get_impute_method}
\title{Get the impute function from either the intervals column or from the method}
\usage{
get_impute_method(intervals, impute)
}
\arguments{
\item{intervals}{the data.frame of intervals}

\item{impute}{the imputation definition}
}
\value{
The imputation function vector
}
\description{
Get the impute function from either the intervals column or from the method
}
