% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/set_and_assert_intervals.R
\name{assert_intervals}
\alias{assert_intervals}
\title{Assert Intervals}
\usage{
assert_intervals(intervals, data)
}
\arguments{
\item{intervals}{Proposed intervals}

\item{data}{PKNCAdata object}
}
\value{
The intervals argument unchanged, or it will raise an error.
}
\description{
Verifies that an interval definition is valid for a PKNCAdata object.
Valid means that intervals are a data.frame (or data.frame-like object),
that the column names are either the groupings of the PKNCAconc part of
the PKNCAdata object or that they are one of the NCA parameters allowed
(i.e. names(get.interval.cols())).
It will return the intervals argument unchanged, or it will raise an error.
}
