% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parse_formula_to_cols.R
\name{parse_formula_to_cols}
\alias{parse_formula_to_cols}
\title{Convert a formula representation to the columns for input data}
\usage{
parse_formula_to_cols(form)
}
\arguments{
\item{form}{the formula (or something coercible into a formula) to extract
into its parts}
}
\value{
A list of column names for various formula parts
}
\description{
Convert a formula representation to the columns for input data
}
\seealso{
Other Formula parsing: 
\code{\link{findOperator}()}
}
\concept{Formula parsing}
\keyword{Internal}
