% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pk.calc.simple.R
\name{pk.calc.vss}
\alias{pk.calc.vss}
\title{Calculate the steady-state volume of distribution (Vss)}
\usage{
pk.calc.vss(cl, mrt)
}
\arguments{
\item{cl}{the clearance}

\item{mrt}{the mean residence time}
}
\value{
the volume of distribution at steady-state
}
\description{
Calculate the steady-state volume of distribution (Vss)
}
\details{
vss is \code{cl*mrt}.
}
