% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pk.calc.simple.R
\name{pk.calc.cav}
\alias{pk.calc.cav}
\title{Calculate the average concentration during an interval.}
\usage{
pk.calc.cav(auc, start, end)
}
\arguments{
\item{auc}{The area under the curve during the interval}

\item{start}{The start time of the interval}

\item{end}{The end time of the interval}
}
\value{
The Cav (average concentration during the interval)
}
\description{
Calculate the average concentration during an interval.
}
\details{
cav is \code{auc/(end-start)}.
}
