% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pk.calc.simple.R
\name{pk.calc.thalf.eff}
\alias{pk.calc.thalf.eff}
\title{Calculate the effective half-life}
\usage{
pk.calc.thalf.eff(mrt)
}
\arguments{
\item{mrt}{the mean residence time to infinity}
}
\value{
the numeric value of the effective half-life
}
\description{
Calculate the effective half-life
}
\details{
thalf.eff is \code{log(2)*mrt}.
}
